@@ -503,8 +503,8 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
 /*************            Other functions          *************/
 /***************************************************************/
 static int
-arithmetic_binary_conn_sanity_checks(gal_data_t *in, gal_data_t *conn,
-                                     char *operator)
+arithmetic_binary_sanity_checks(gal_data_t *in, gal_data_t *conn,
+                                char *operator)
 {
   int conn_int;
 
@@ -552,19 +552,14 @@ arithmetic_erode_dilate(struct arithmeticparams *p, char *token, int op)
   gal_data_t *conn = operands_pop(p, token);
   gal_data_t *in   = operands_pop(p, token);
 
-  /* Do the sanity checks and  */
+  /* Do the sanity checks. */
+  conn_int=arithmetic_binary_sanity_checks(in, conn, token);
+
+  /* Do the operation. */
   switch(op)
     {
-    case ARITHMETIC_OP_ERODE:
-      conn_int=arithmetic_binary_conn_sanity_checks(in, conn, "erode");
-      gal_binary_erode(in, 1, conn_int, 1);
-      break;
-
-    case ARITHMETIC_OP_DILATE:
-      conn_int=arithmetic_binary_conn_sanity_checks(in, conn, "dilate");
-      gal_binary_dilate(in, 1, conn_int, 1);
-      break;
-
+    case ARITHMETIC_OP_ERODE:  gal_binary_erode(in,  1, conn_int, 1); break;
+    case ARITHMETIC_OP_DILATE: gal_binary_dilate(in, 1, conn_int, 1); break;
     default:
       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
             "problem. The operator code %d not recognized", __func__,
@@ -573,8 +568,6 @@ arithmetic_erode_dilate(struct arithmeticparams *p, char *token, int op)
 
   /* Push the result onto the stack. */
   operands_add(p, NULL, in);
-
-  /* Recall that`conn' was freed in the sanity check. */
 }
 
 
@@ -592,8 +585,7 @@ arithmetic_connected_components(struct arithmeticparams *p, char *token)
   gal_data_t *in   = operands_pop(p, token);
 
   /* Basic sanity checks. */
-  conn_int=arithmetic_binary_conn_sanity_checks(in, conn,
-                                                "connected-components");
+  conn_int=arithmetic_binary_sanity_checks(in, conn, token);
 
   /* Do the connected components labeling. */
   gal_binary_connected_components(in, &out, conn_int);
@@ -609,6 +601,29 @@ arithmetic_connected_components(struct arithmeticparams *p, char *token)
 
 
 
+static void
+arithmetic_fill_holes(struct arithmeticparams *p, char *token)
+{
+  int conn_int;
+
+  /* Pop the two necessary operands. */
+  gal_data_t *conn = operands_pop(p, token);
+  gal_data_t *in   = operands_pop(p, token);
+
+  /* Basic sanity checks. */
+  conn_int=arithmetic_binary_sanity_checks(in, conn, token);
+
+  /* Fill the holes */
+  gal_binary_holes_fill(in, conn_int, -1);
+
+  /* Push the result onto the stack. */
+  operands_add(p, NULL, in);
+}
+
+
+
+
+
 static void
 arithmetic_invert(struct arithmeticparams *p, char *token)
 {
@@ -685,6 +700,99 @@ arithmetic_interpolate(struct arithmeticparams *p, char *token)
 
 
 
+static void
+arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
+{
+  long dim;
+  gal_data_t *collapsed=NULL;
+
+  /* First popped operand is the dimension. */
+  gal_data_t *dimension = operands_pop(p, token);
+
+  /* The second popped operand is the desired input dataset. */
+  gal_data_t *input = operands_pop(p, token);
+
+
+  /* Small sanity check. */
+  if( dimension->ndim!=1 || dimension->size!=1)
+    error(EXIT_FAILURE, 0, "First popped operand of `collapse-*' operators "
+          "(dimension to collapse) must be a single number (single-element, "
+          "one-dimensional dataset). But it has %zu dimension(s) and %zu "
+          "element(s).", dimension->ndim, dimension->size);
+  if(dimension->type==GAL_TYPE_FLOAT32 || dimension->type==GAL_TYPE_FLOAT64)
+    error(EXIT_FAILURE, 0, "First popped operand of `collapse-*' operators "
+          "(dimension to collapse) must have an integer type, but it has "
+          "a floating point type (`%s')", gal_type_name(dimension->type,1));
+  dimension=gal_data_copy_to_new_type_free(dimension, GAL_TYPE_LONG);
+  dim=((long *)(dimension->array))[0];
+  if(dim<0 || dim==0)
+    error(EXIT_FAILURE, 0, "First popped operand of `collapse-*' operators "
+          "(dimension to collapse) must be positive (larger than zero), it "
+          "is %ld", dim);
+
+
+  /* If a WCS structure has been read, we'll need to pass it to
+     `gal_dimension_collapse', so it modifies it respectively. */
+  input->wcs=p->refdata.wcs;
+
+
+  /* Run the relevant library function. */
+  switch(operator)
+    {
+    case ARITHMETIC_OP_COLLAPSE_SUM:
+      collapsed=gal_dimension_collapse_sum(input, input->ndim-dim, NULL);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MEAN:
+      collapsed=gal_dimension_collapse_mean(input, input->ndim-dim, NULL);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_NUMBER:
+      collapsed=gal_dimension_collapse_number(input, input->ndim-dim);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MIN:
+      collapsed=gal_dimension_collapse_minmax(input, input->ndim-dim, 0);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_MAX:
+      collapsed=gal_dimension_collapse_minmax(input, input->ndim-dim, 1);
+      break;
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
+            "problem. The operator code %d is not recognized", __func__,
+            PACKAGE_BUGREPORT, operator);
+    }
+
+
+  /* If a WCS structure existed, a modified WCS is now present in
+     `collapsed->wcs'. So we'll let the freeing of `input' free the old
+     `p->refdata.wcs' structure and we'll put the new one there, then we'll
+     set `collapsed->wcs' to `NULL', so the new one isn't freed. */
+  p->refdata.wcs = collapsed->wcs;
+  collapsed->wcs = NULL;
+
+
+  /* We'll also need to correct the size of the reference dataset. We'll
+     use `memcpy' to write the new `dsize' values into the old ones. The
+     dimensions have decreased, so we won't be writing outside of allocated
+     space that `p->refdata.dsize' points to. */
+  p->refdata.ndim -= 1;
+  memcpy( p->refdata.dsize, collapsed->dsize,
+           p->refdata.ndim * (sizeof *p->refdata.dsize) );
+
+
+  /* Clean up and add the collapsed dataset to the top of the operands. */
+  gal_data_free(input);
+  gal_data_free(dimension);
+  operands_add(p, NULL, collapsed);
+}
+
+
+
+
+
 
 
 
@@ -732,10 +840,14 @@ reversepolish(struct arithmeticparams *p)
       /* If we have a name or number, then add it to the operands linked
          list. Otherwise, pull out two members and do the specified
          operation on them. */
-      if(gal_array_name_recognized(token->v))
+      if( gal_array_name_recognized(token->v)
+          || operands_is_name(p, token->v) )
         operands_add(p, token->v, NULL);
       else if( (d1=gal_data_copy_string_to_number(token->v)) )
         operands_add(p, NULL, d1);
+      else if( !strncmp(token->v, SET_OPERATOR_PREFIX,
+                        SET_OPERATOR_PREFIX_LENGTH) )
+        operands_set_name(p, token->v);
       else
         {
 
@@ -854,7 +966,7 @@ reversepolish(struct arithmeticparams *p)
           else if (!strcmp(token->v, "float64"))
             { op=GAL_ARITHMETIC_OP_TO_FLOAT64;        nop=1;  }
 
-          /* Filters. */
+          /* Library wrappers. */
           else if (!strcmp(token->v, "filter-mean"))
             { op=ARITHMETIC_OP_FILTER_MEAN;           nop=0;  }
           else if (!strcmp(token->v, "filter-median"))
@@ -869,16 +981,28 @@ reversepolish(struct arithmeticparams *p)
             { op=ARITHMETIC_OP_DILATE;                nop=0;  }
           else if (!strcmp(token->v, "connected-components"))
             { op=ARITHMETIC_OP_CONNECTED_COMPONENTS;  nop=0;  }
+          else if (!strcmp(token->v, "fill-holes"))
+            { op=ARITHMETIC_OP_FILL_HOLES;            nop=0;  }
           else if (!strcmp(token->v, "invert"))
             { op=ARITHMETIC_OP_INVERT;                nop=0;  }
           else if (!strcmp(token->v, "interpolate-medianngb"))
             { op=ARITHMETIC_OP_INTERPOLATE_MEDIANNGB; nop=0;  }
+          else if (!strcmp(token->v, "collapse-sum"))
+            { op=ARITHMETIC_OP_COLLAPSE_SUM;          nop=0; }
+          else if (!strcmp(token->v, "collapse-min"))
+            { op=ARITHMETIC_OP_COLLAPSE_MIN;          nop=0; }
+          else if (!strcmp(token->v, "collapse-max"))
+            { op=ARITHMETIC_OP_COLLAPSE_MAX;          nop=0; }
+          else if (!strcmp(token->v, "collapse-mean"))
+            { op=ARITHMETIC_OP_COLLAPSE_MEAN;         nop=0; }
+          else if (!strcmp(token->v, "collapse-number"))
+            { op=ARITHMETIC_OP_COLLAPSE_NUMBER;       nop=0; }
 
 
           /* Finished checks with known operators */
           else
             error(EXIT_FAILURE, 0, "the argument \"%s\" could not be "
-                  "interpretted as a recognized input file name, number, or "
+                  "interpretted as a file name, named dataset, number, or "
                   "operator", token->v);
 
 
@@ -956,6 +1080,10 @@ reversepolish(struct arithmeticparams *p)
                   arithmetic_connected_components(p, token->v);
                   break;
 
+                case ARITHMETIC_OP_FILL_HOLES:
+                  arithmetic_fill_holes(p, token->v);
+                  break;
+
                 case ARITHMETIC_OP_INVERT:
                   arithmetic_invert(p, token->v);
                   break;
@@ -964,6 +1092,14 @@ reversepolish(struct arithmeticparams *p)
                   arithmetic_interpolate(p, token->v);
                   break;
 
+                case ARITHMETIC_OP_COLLAPSE_SUM:
+                case ARITHMETIC_OP_COLLAPSE_MIN:
+                case ARITHMETIC_OP_COLLAPSE_MAX:
+                case ARITHMETIC_OP_COLLAPSE_MEAN:
+                case ARITHMETIC_OP_COLLAPSE_NUMBER:
+                  arithmetic_collapse(p, token->v, op);
+                  break;
+
                 default:
                   error(EXIT_FAILURE, 0, "%s: a bug! please contact us at "
                         "%s to fix the problem. The code %d is not "
@@ -972,8 +1108,18 @@ reversepolish(struct arithmeticparams *p)
                 }
             }
         }
+
+      /* Increment the token counter. */
+      ++p->tokencounter;
     }
 
+
+  /* If there aren't any more operands (a variable has been set but not
+     used), then there is nothing to create. */
+  if(p->operands==NULL)
+    error(EXIT_FAILURE, 0, "no operands on the stack to write (as output)");
+
+
   /* If there is more than one node in the operands stack then the user has
      given too many operands which is an error. */
   if(p->operands->next!=NULL)
@@ -1033,6 +1179,7 @@ reversepolish(struct arithmeticparams *p)
      into `d1', so it is freed when freeing d1. */
   gal_data_free(d1);
   free(p->refdata.dsize);
+  gal_list_data_free(p->named);
 
   /* Clean up. Note that the tokens were taken from the command-line
      arguments, so the string within each token linked list must not be
