@@ -38,8 +38,14 @@ enum arithmetic_prog_operators
   ARITHMETIC_OP_ERODE,
   ARITHMETIC_OP_DILATE,
   ARITHMETIC_OP_CONNECTED_COMPONENTS,
+  ARITHMETIC_OP_FILL_HOLES,
   ARITHMETIC_OP_INVERT,
   ARITHMETIC_OP_INTERPOLATE_MEDIANNGB,
+  ARITHMETIC_OP_COLLAPSE_SUM,
+  ARITHMETIC_OP_COLLAPSE_MIN,
+  ARITHMETIC_OP_COLLAPSE_MAX,
+  ARITHMETIC_OP_COLLAPSE_MEAN,
+  ARITHMETIC_OP_COLLAPSE_NUMBER,
 };
 
 
