@@ -3,9 +3,9 @@ Arithmetic - Do arithmetic operations on images.
 Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,8 +31,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/wcs.h>
 #include <gnuastro/box.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/array.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/table.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/options.h>
@@ -655,8 +657,8 @@ ui_read_cols(struct mkprofparams *p)
         case 3:
           if(tmp->type==GAL_TYPE_STRING)
             {
-              p->f=gal_data_malloc_array(GAL_TYPE_UINT8, p->num,
-                                         __func__, "p->f");
+              p->f=gal_pointer_allocate(GAL_TYPE_UINT8, p->num, 0,
+                                        __func__, "p->f");
               strarr=tmp->array;
               for(i=0;i<p->num;++i)
                 p->f[i]=ui_profile_name_read(strarr[i], i+1);
@@ -801,15 +803,15 @@ ui_prepare_columns(struct mkprofparams *p)
       p->num=1;
 
       /* Allocate the necessary columns. */
-      p->x = gal_data_calloc_array(GAL_TYPE_FLOAT64, 1, __func__, "p->x");
-      p->y = gal_data_calloc_array(GAL_TYPE_FLOAT64, 1, __func__, "p->y");
-      p->f = gal_data_calloc_array(GAL_TYPE_UINT8,   1, __func__, "p->f");
-      p->r = gal_data_calloc_array(GAL_TYPE_FLOAT32, 1, __func__, "p->r");
-      p->n = gal_data_calloc_array(GAL_TYPE_FLOAT32, 1, __func__, "p->n");
-      p->p = gal_data_calloc_array(GAL_TYPE_FLOAT32, 1, __func__, "p->p");
-      p->q = gal_data_calloc_array(GAL_TYPE_FLOAT32, 1, __func__, "p->q");
-      p->m = gal_data_calloc_array(GAL_TYPE_FLOAT32, 1, __func__, "p->m");
-      p->t = gal_data_calloc_array(GAL_TYPE_FLOAT32, 1, __func__, "p->t");
+      p->x = gal_pointer_allocate(GAL_TYPE_FLOAT64, 1, 1, __func__, "p->x");
+      p->y = gal_pointer_allocate(GAL_TYPE_FLOAT64, 1, 1, __func__, "p->y");
+      p->f = gal_pointer_allocate(GAL_TYPE_UINT8,   1, 1, __func__, "p->f");
+      p->r = gal_pointer_allocate(GAL_TYPE_FLOAT32, 1, 1, __func__, "p->r");
+      p->n = gal_pointer_allocate(GAL_TYPE_FLOAT32, 1, 1, __func__, "p->n");
+      p->p = gal_pointer_allocate(GAL_TYPE_FLOAT32, 1, 1, __func__, "p->p");
+      p->q = gal_pointer_allocate(GAL_TYPE_FLOAT32, 1, 1, __func__, "p->q");
+      p->m = gal_pointer_allocate(GAL_TYPE_FLOAT32, 1, 1, __func__, "p->m");
+      p->t = gal_pointer_allocate(GAL_TYPE_FLOAT32, 1, 1, __func__, "p->t");
 
       /* For profiles that need a different number of input values. Note
          that when a profile doesn't need a value, it will be ignored. */
@@ -1030,9 +1032,11 @@ ui_prepare_canvas(struct mkprofparams *p)
       else
         {
           /* Read the image. */
-          p->out=gal_fits_img_read_to_type(p->backname, p->backhdu,
-                                           GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize, 0, 0);
+          p->out=gal_array_read_one_ch_to_type(p->backname, p->backhdu,
+                                               GAL_TYPE_FLOAT32,
+                                               p->cp.minmapsize);
+          p->out->wcs=gal_wcs_read(p->backname, p->backhdu, 0, 0,
+                                   &p->out->nwcs);
 
           /* Put the WCS structure and number of dimensions in the
              MakeProfiles's main structure for generality. The WCS
@@ -1048,8 +1052,8 @@ ui_prepare_canvas(struct mkprofparams *p)
           if( p->dsize ) free(p->dsize);
 
           /* Write the size of the background image into `dsize'. */
-          p->dsize=gal_data_malloc_array(GAL_TYPE_SIZE_T, p->ndim, __func__,
-                                         "p->dsize");
+          p->dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->ndim, 0,
+                                        __func__, "p->dsize");
           for(i=0;i<p->ndim;++i) p->dsize[i] = p->out->dsize[i];
 
           /* Set all pixels to zero if the user wanted a clear canvas. */
@@ -1069,8 +1073,8 @@ ui_prepare_canvas(struct mkprofparams *p)
          there is no shifts. */
       p->oversample=1;
       if(p->shift) free(p->shift);
-      p->shift=gal_data_calloc_array(GAL_TYPE_SIZE_T, p->ndim, __func__,
-                                     "p->shift (1)");
+      p->shift=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->ndim, 1, __func__,
+                                    "p->shift (1)");
     }
   else
     {
@@ -1130,8 +1134,8 @@ ui_prepare_canvas(struct mkprofparams *p)
                  shifts (from zero). So, we'll just free it and reset
                  it. */
               if(p->shift) free(p->shift);
-              p->shift=gal_data_calloc_array(GAL_TYPE_SIZE_T, p->ndim,
-                                             __func__, "p->shift (2)");
+              p->shift=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->ndim, 1,
+                                            __func__, "p->shift (2)");
               if(setshift)
                 {
                   p->shift[0]  = (width[0]/2)*p->oversample;
@@ -1142,8 +1146,8 @@ ui_prepare_canvas(struct mkprofparams *p)
 
       /* If shift has not been set until now, set it. */
       if(p->shift==NULL)
-        p->shift=gal_data_calloc_array(GAL_TYPE_SIZE_T, p->ndim, __func__,
-                                       "p->shift (3)");
+        p->shift=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->ndim, 1,
+                                      __func__, "p->shift (3)");
 
       /* Prepare the sizes of the final merged image (if it is to be
          made). Note that even if we don't want a merged image, we still
@@ -1316,7 +1320,8 @@ ui_make_log(struct mkprofparams *p)
 
   /* Row number in input catalog. */
   name=gal_fits_name_save_as_string(p->catname, p->cp.hdu);
-  asprintf(&comment, "Row number of profile in %s.", name);
+  if( asprintf(&comment, "Row number of profile in %s.", name)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_data_add_alloc(&p->log, NULL, GAL_TYPE_UINT64, 1, &p->num, NULL,
                           1, p->cp.minmapsize, "INPUT_ROW_NO", "count",
                           comment);
@@ -1342,8 +1347,8 @@ ui_preparations(struct mkprofparams *p)
       p->individual=1;
 
       /* Set the shift array. */
-      p->shift=gal_data_calloc_array(GAL_TYPE_SIZE_T, p->ndim,
-                                     __func__, "p->shift");
+      p->shift=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->ndim, 1,
+                                    __func__, "p->shift");
     }
   else
     ui_prepare_canvas(p);
@@ -1395,40 +1400,56 @@ ui_print_intro(struct mkprofparams *p)
   printf(PROGRAM_NAME" started on %s", ctime(&p->rawtime));
 
   if(p->kernel)
-    asprintf(&jobname, "Building one %s kernel",
-             ui_profile_name_write(p->kernel->status));
+    {
+      if( asprintf(&jobname, "Building one %s kernel",
+                   ui_profile_name_write(p->kernel->status))<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
   else
-    asprintf(&jobname, "%zu profile%sread from %s", p->num,
-             p->num>1?"s ":" ", p->catname);
+    {
+      if( asprintf(&jobname, "%zu profile%sread from %s", p->num,
+                   p->num>1?"s ":" ", p->catname)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
   gal_timing_report(NULL, jobname, 1);
   free(jobname);
 
   if(p->backname)
     {
       if(p->nomerged)
-        asprintf(&jobname, "WCS information read from %s", p->backname);
+        {
+          if( asprintf(&jobname, "WCS information read from %s",
+                       p->backname)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       else
-        asprintf(&jobname, "%s is read and will be used as canvas",
-                 p->backname);
+        {
+          if( asprintf(&jobname, "%s is read and will be used as canvas",
+                       p->backname)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       gal_timing_report(NULL, jobname, 1);
       free(jobname);
     }
 
-  asprintf(&jobname, "Random number generator (RNG) type: %s",
-           gsl_rng_name(p->rng));
+  if( asprintf(&jobname, "Random number generator (RNG) type: %s",
+               gsl_rng_name(p->rng))<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_timing_report(NULL, jobname, 1);
   free(jobname);
   if(p->envseed)
     {
-      asprintf(&jobname, "RNG seed for all profiles: %lu",
-               gsl_rng_default_seed);
+      if( asprintf(&jobname, "RNG seed for all profiles: %lu",
+                   gsl_rng_default_seed)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(NULL, jobname, 1);
       free(jobname);
     }
 
   if(p->kernel==NULL)
     {
-      asprintf(&jobname, "Using %zu threads.", p->cp.numthreads);
+      if( asprintf(&jobname, "Using %zu threads.", p->cp.numthreads)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(NULL, jobname, 1);
       free(jobname);
     }
