@@ -1,11 +1,11 @@
 /*********************************************************************
-NoiseChisel - Detect and segment signal in a noisy dataset.
+NoiseChisel - Detect signal in a noisy dataset.
 NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,6 +31,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/fits.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/statistics.h>
 #include <gnuastro/interpolate.h>
 
@@ -196,10 +197,17 @@ threshold_write_sn_table(struct noisechiselparams *p, gal_data_t *insn,
      set of blank elements, but checking on the integer array is faster. */
   if( gal_blank_present(inind, 1) )
     {
+      /* Remove blank elements. */
       ind=gal_data_copy(inind);
       sn=gal_data_copy(insn);
       gal_blank_remove(ind);
       gal_blank_remove(sn);
+
+      /* A small sanity check. */
+      if(ind->size==0 || sn->size==0)
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+              "the problem. For some reason, all the elements in `ind' or "
+              "`sn' are blank", __func__, PACKAGE_BUGREPORT);
     }
   else
     {
@@ -218,7 +226,7 @@ threshold_write_sn_table(struct noisechiselparams *p, gal_data_t *insn,
 
   /* write the table. */
   gal_checkset_writable_remove(filename, 0, 1);
-  gal_table_write(cols, comments, p->cp.tableformat, filename, "SN");
+  gal_table_write(cols, comments, p->cp.tableformat, filename, "SN", 0);
 
 
   /* Clean up (if necessary). */
@@ -393,8 +401,8 @@ qthresh_on_tile(void *in_prm)
 
   /* Put the temporary usage space for this thread into a data set for easy
      processing. */
-  usage=gal_data_alloc(gal_data_ptr_increment(qprm->usage,
-                                              tprm->id*p->maxtcontig, type),
+  usage=gal_data_alloc(gal_pointer_increment(qprm->usage,
+                                             tprm->id*p->maxtcontig, type),
                        type, ndim, p->maxtsize, NULL, 0, p->cp.minmapsize,
                        NULL, NULL, NULL);
 
@@ -455,13 +463,13 @@ qthresh_on_tile(void *in_prm)
           /* Get the erosion quantile for this tile and save it. Note that
              the type of `qvalue' is the same as the input dataset. */
           qvalue=gal_statistics_quantile(usage, p->qthresh, 1);
-          memcpy(gal_data_ptr_increment(qprm->erode_th->array, tind, type),
+          memcpy(gal_pointer_increment(qprm->erode_th->array, tind, type),
                  qvalue->array, twidth);
           gal_data_free(qvalue);
 
           /* Same for the no-erode quantile. */
           qvalue=gal_statistics_quantile(usage, p->noerodequant, 1);
-          memcpy(gal_data_ptr_increment(qprm->noerode_th->array, tind, type),
+          memcpy(gal_pointer_increment(qprm->noerode_th->array, tind, type),
                  qvalue->array, twidth);
           gal_data_free(qvalue);
 
@@ -469,7 +477,7 @@ qthresh_on_tile(void *in_prm)
           if(p->detgrowquant!=1.0f)
             {
               qvalue=gal_statistics_quantile(usage, p->detgrowquant, 1);
-              memcpy(gal_data_ptr_increment(qprm->expand_th->array, tind,
+              memcpy(gal_pointer_increment(qprm->expand_th->array, tind,
                                             type),
                      qvalue->array, twidth);
               gal_data_free(qvalue);
@@ -477,12 +485,12 @@ qthresh_on_tile(void *in_prm)
         }
       else
         {
-          gal_blank_write(gal_data_ptr_increment(qprm->erode_th->array,
+          gal_blank_write(gal_pointer_increment(qprm->erode_th->array,
                                                  tind, type), type);
-          gal_blank_write(gal_data_ptr_increment(qprm->noerode_th->array,
+          gal_blank_write(gal_pointer_increment(qprm->noerode_th->array,
                                                  tind, type), type);
           if(p->detgrowquant!=1.0f)
-            gal_blank_write(gal_data_ptr_increment(qprm->expand_th->array,
+            gal_blank_write(gal_pointer_increment(qprm->expand_th->array,
                                                    tind, type), type);
         }
 
@@ -531,11 +539,11 @@ threshold_qthresh_clean_work(struct noisechiselparams *p, gal_data_t *first,
       if(third) oa3=third->array;
 
       /* Increment the array pointers. */
-      first->array=gal_data_ptr_increment(first->array, start, first->type);
-      second->array=gal_data_ptr_increment(second->array, start,
+      first->array=gal_pointer_increment(first->array, start, first->type);
+      second->array=gal_pointer_increment(second->array, start,
                                            second->type);
       if(third)
-        third->array=gal_data_ptr_increment(third->array, start, third->type);
+        third->array=gal_pointer_increment(third->array, start, third->type);
 
       /* Correct their sizes. */
       first->size=number;
@@ -648,6 +656,8 @@ void
 threshold_quantile_find_apply(struct noisechiselparams *p)
 {
   char *msg;
+  size_t nval;
+  gal_data_t *num;
   struct timeval t1;
   struct qthreshparams qprm;
   struct gal_options_common_params *cp=&p->cp;
@@ -688,9 +698,9 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
 
 
   /* Allocate temporary space for processing in each tile. */
-  qprm.usage=gal_data_malloc_array(p->input->type,
-                                   cp->numthreads * p->maxtcontig,
-                                   __func__, "qprm.usage");
+  qprm.usage=gal_pointer_allocate(p->input->type,
+                                  cp->numthreads * p->maxtcontig, 0,
+                                  __func__, "qprm.usage");
 
 
   /* Find the threshold on each tile, free the temporary processing space
@@ -734,6 +744,38 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
                             p->qthreshname);
 
 
+  /* Check if the number of acceptable tiles is more than the minimum
+     interpolated number. Since this is a common problem for users, it is
+     much more useful to do the check here rather than printing multiple
+     errors in parallel. */
+  num=gal_statistics_number(qprm.erode_th);
+  nval=((size_t *)(num->array))[0];
+  if( nval < cp->interpnumngb)
+    error(EXIT_FAILURE, 0, "%zu tiles can be used for interpolation of the "
+          "quantile threshold values over the full dataset. This is smaller "
+          "than the requested minimum value of %zu (value to the "
+          "`--interpnumngb' option).\n\n"
+          "There are several ways to address the problem. The best and most "
+          "highly recommended is to use a larger input if possible (when the "
+          "input is a crop from a larger dataset). If that is not the case, "
+          "or it doesn't solve the problem, you need to loosen the "
+          "parameters (and therefore cause scatter in the final result). "
+          "Thus don't loosen them too much. Recall that you can see all the "
+          "option values to Gnuastro's programs by appending `-P' to the "
+          "end of your command.\n\n"
+          "  * Decrease `--interpnumngb' to be smaller than %zu.\n"
+          "  * Slightly decrease `--tilesize' to have more tiles.\n"
+          "  * Slightly increase `--modmedqdiff' to accept more tiles.\n\n"
+          "Try appending your command with `--checkqthresh' to see the "
+          "successful tiles (and get a feeling of the cause/solution. Note "
+          "that the output is a multi-extension FITS file).\n\n"
+          "To better understand this important step, please run the "
+          "following command (press `SPACE'/arrow-keys to navigate and "
+          "`Q' to return back to the command-line):\n\n"
+          "    $ info gnuastro \"Quantifying signal in a tile\"\n",
+          nval, cp->interpnumngb, cp->interpnumngb);
+
+
   /* Interpolate and smooth the derived values. */
   threshold_interp_smooth(p, &qprm.erode_th, &qprm.noerode_th,
                           qprm.expand_th ? &qprm.expand_th : NULL,
@@ -747,7 +789,11 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
 
   /* Write the binary image if check is requested. */
   if(p->qthreshname && !tl->oneelempertile)
-    gal_fits_img_write(p->binary, p->qthreshname, NULL, PROGRAM_NAME);
+    {
+      p->binary->name="QTHRESH-APPLIED";
+      gal_fits_img_write(p->binary, p->qthreshname, NULL, PROGRAM_NAME);
+      p->binary->name=NULL;
+    }
 
 
   /* Set the expansion quantile if necessary. */
@@ -759,8 +805,9 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
   gal_data_free(qprm.noerode_th);
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "%.2f & %0.2f quantile thresholds applied.",
-               p->qthresh, p->noerodequant);
+      if( asprintf(&msg, "%.2f & %0.2f quantile thresholds applied.",
+                   p->qthresh, p->noerodequant)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
     }
