@@ -3,9 +3,9 @@ Statistics - Statistical analysis on input dataset.
 Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,6 +30,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/tile.h>
+#include <gnuastro/array.h>
 #include <gnuastro/qsort.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/table.h>
@@ -296,8 +297,16 @@ ui_read_quantile_range(struct argp_option *option, char *arg,
   /* For the `--printparams' (`-P') option:*/
   if(lineno==-1)
     {
-      if( isnan(p->quantmax) ) asprintf(&str, "%g", p->quantmin);
-      else     asprintf(&str, "%g,%g", p->quantmin, p->quantmax);
+      if( isnan(p->quantmax) )
+        {
+          if( asprintf(&str, "%g", p->quantmin)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&str, "%g,%g", p->quantmin, p->quantmax)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       return str;
     }
 
@@ -524,7 +533,8 @@ ui_check_options_and_arguments(struct statisticsparams *p)
                       "to tables.", p->inputname, p->cp.hdu);
             }
           else if(p->column==NULL)
-            asprintf(&name, "%s (hdu: %s)", p->inputname, p->cp.hdu);
+            if( asprintf(&name, "%s (hdu: %s)", p->inputname, p->cp.hdu)<0 )
+              error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
         }
 
       /* If its not FITS, it must be a table. */
@@ -645,7 +655,7 @@ ui_out_of_range_to_blank(struct statisticsparams *p)
   /* Allocate a blank value to mask all pixels that don't satisfy the
      condition. */
   blank=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL,
-                     0, -1, NULL, NULL, NULL);
+                       0, -1, NULL, NULL, NULL);
   *((float *)(blank->array)) = NAN;
 
 
@@ -653,6 +663,11 @@ ui_out_of_range_to_blank(struct statisticsparams *p)
      blank value will be used in the proper type of the input in the
      `where' operator.*/
   gal_arithmetic(GAL_ARITHMETIC_OP_WHERE, flagsor, p->input, cond, blank);
+
+
+  /* Reset the blank flags so they are checked again if necessary. */
+  p->input->flag &= ~GAL_DATA_FLAG_BLANK_CH;
+  p->input->flag &= ~GAL_DATA_FLAG_HASBLANK;
 }
 
 
@@ -689,7 +704,7 @@ ui_make_sorted_if_necessary(struct statisticsparams *p)
      point it to the input.*/
   if(is_necessary)
     {
-      if( gal_statistics_is_sorted(p->input) )
+      if( gal_statistics_is_sorted(p->input, 1) )
         p->sorted=p->input;
       else
         {
@@ -756,11 +771,12 @@ ui_read_columns(struct statisticsparams *p)
 
       /* Print an error if there are too many columns: */
       if(toomanycols)
-        gal_tableintern_error_col_selection(p->inputname, p->cp.hdu, "too many "
-                                            "columns were selected by the "
-                                            "given values to the `--column' "
-                                            "and/or `--refcol' options. Only "
-                                            "one is acceptable for each.");
+        gal_tableintern_error_col_selection(p->inputname, p->cp.hdu, "too "
+                                            "many columns were selected by "
+                                            "the given values to the "
+                                            "`--column' and/or `--refcol' "
+                                            "options. Only one is "
+                                            "acceptable for each.");
     }
 
   /* Clean up. */
@@ -775,14 +791,21 @@ void
 ui_preparations(struct statisticsparams *p)
 {
   gal_data_t *check;
+  int keepinputdir=p->cp.keepinputdir;
   struct gal_options_common_params *cp=&p->cp;
   struct gal_tile_two_layer_params *tl=&cp->tl;
+  char *checkbasename = p->cp.output ? p->cp.output : p->inputname;
+
+  /* Change `keepinputdir' based on if an output name was given. */
+  p->cp.keepinputdir = p->cp.output ? 1 : 0;
 
   /* Read the input. */
   if(p->isfits && p->hdu_type==IMAGE_HDU)
     {
       p->inputformat=INPUT_FORMAT_IMAGE;
-      p->input=gal_fits_img_read(p->inputname, cp->hdu, cp->minmapsize, 0, 0);
+      p->input=gal_array_read_one_ch(p->inputname, cp->hdu, cp->minmapsize);
+      p->input->wcs=gal_wcs_read(p->inputname, cp->hdu, 0, 0,
+                                 &p->input->nwcs);
     }
   else
     {
@@ -806,7 +829,7 @@ ui_preparations(struct statisticsparams *p)
       /* Make the tile check image if requested. */
       if(tl->checktiles)
         {
-          tl->tilecheckname=gal_checkset_automatic_output(cp, p->inputname,
+          tl->tilecheckname=gal_checkset_automatic_output(cp, checkbasename,
                                                           "_tiled.fits");
           check=gal_tile_block_check_tiles(tl->tiles);
           if(p->inputformat==INPUT_FORMAT_IMAGE)
@@ -816,14 +839,14 @@ ui_preparations(struct statisticsparams *p)
               gal_checkset_writable_remove(tl->tilecheckname, 0,
                                            cp->dontdelete);
               gal_table_write(check, NULL, cp->tableformat, tl->tilecheckname,
-                              "TABLE");
+                              "TABLE", 0);
             }
           gal_data_free(check);
         }
 
       /* Set the steps image name. */
       if(p->sky && p->checksky)
-        p->checkskyname=gal_checkset_automatic_output(cp, p->inputname,
+        p->checkskyname=gal_checkset_automatic_output(cp, checkbasename,
                                                       "_sky_steps.fits");
     }
 
@@ -835,6 +858,12 @@ ui_preparations(struct statisticsparams *p)
     {
       /* Only keep the elements we want. */
       gal_blank_remove(p->input);
+
+      /* Make sure there actually are any (non-blank) elements left. */
+      if(p->input->size==0)
+        error(EXIT_FAILURE, 0, "%s: all elements are blank",
+              gal_fits_name_save_as_string(p->inputname, cp->hdu));
+
       p->input->flag &= ~GAL_DATA_FLAG_HASBLANK ;
       p->input->flag |= GAL_DATA_FLAG_BLANK_CH ;
 
@@ -851,6 +880,9 @@ ui_preparations(struct statisticsparams *p)
       if( !isnan(p->mirror) )             ++p->numoutfiles;
       if( p->histogram || p->cumulative ) ++p->numoutfiles;
     }
+
+  /* Reset `keepinputdir' to what it originally was. */
+  p->cp.keepinputdir=keepinputdir;
 }
 
 
