@@ -3,9 +3,9 @@ blank -- Deal with blank values in a datasets
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,6 +32,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/data.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/blank.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/checkset.h>
 
@@ -87,7 +88,7 @@ gal_blank_alloc_write(uint8_t type)
   void *out;
 
   /* Allocate the space to keep the blank value. */
-  out=gal_data_malloc_array(type, 1, __func__, "out");
+  out=gal_pointer_allocate(type, 1, 0, __func__, "out");
 
   /* Put the blank value in the allcated space. */
   gal_blank_write(out, type);
@@ -112,6 +113,57 @@ gal_blank_initialize(gal_data_t *input)
 
 
 
+
+/* Return 1 if the contents of the pointer (with the given type) is
+   blank. */
+int
+gal_blank_is(void *pointer, uint8_t type)
+{
+  switch(type)
+    {
+    /* Numeric types */
+    case GAL_TYPE_UINT8:     return *(uint8_t  *)pointer==GAL_BLANK_UINT8;
+    case GAL_TYPE_INT8:      return *(int8_t   *)pointer==GAL_BLANK_INT8;
+    case GAL_TYPE_UINT16:    return *(uint16_t *)pointer==GAL_BLANK_UINT16;
+    case GAL_TYPE_INT16:     return *(int16_t  *)pointer==GAL_BLANK_INT16;
+    case GAL_TYPE_UINT32:    return *(uint32_t *)pointer==GAL_BLANK_UINT32;
+    case GAL_TYPE_INT32:     return *(int32_t  *)pointer==GAL_BLANK_INT32;
+    case GAL_TYPE_UINT64:    return *(uint64_t *)pointer==GAL_BLANK_UINT64;
+    case GAL_TYPE_INT64:     return *(int64_t  *)pointer==GAL_BLANK_INT64;
+    case GAL_TYPE_FLOAT32:   return isnan( *(float *)(pointer) );
+    case GAL_TYPE_FLOAT64:   return isnan( *(double *)(pointer) );
+
+    /* String. */
+    case GAL_TYPE_STRING:    if(!strcmp(pointer,GAL_BLANK_STRING)) return 1;
+
+    /* Complex types */
+    case GAL_TYPE_COMPLEX32:
+    case GAL_TYPE_COMPLEX64:
+      error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
+            __func__);
+
+    /* Bit. */
+    case GAL_TYPE_BIT:
+      error(EXIT_FAILURE, 0, "%s: bit type datasets are not yet supported",
+            __func__);
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
+            __func__, type);
+    }
+
+  /* Control should not reach here, so print an error if it does, then
+     return a 0 (just to avoid compiler warnings). */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address the "
+        "problem. Control should not reach the end of this function",
+        __func__, PACKAGE_BUGREPORT);
+  return 0;
+}
+
+
+
+
+
 /* Return 1 if the dataset has a blank value and zero if it doesn't. Before
    checking the dataset, this function will look at its flags. If the
    `GAL_DATA_FLAG_HASBLANK' or `GAL_DATA_FLAG_DONT_CHECK_ZERO' bits of
@@ -385,66 +437,154 @@ gal_blank_as_string(uint8_t type, int width)
       break;
 
     case GAL_TYPE_STRING:
-      if(width) asprintf(&blank, "%*s", width,  GAL_BLANK_STRING);
-      else      asprintf(&blank, "%s",          GAL_BLANK_STRING);
+      if(width)
+        {
+          if( asprintf(&blank, "%*s", width,  GAL_BLANK_STRING)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, "%s", GAL_BLANK_STRING)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_UINT8:
       fmt = width ? "%*"PRIu8 : "%"PRIu8;
-      if(width) asprintf(&blank, fmt, width, (uint8_t)GAL_BLANK_UINT8);
-      else      asprintf(&blank, fmt,        (uint8_t)GAL_BLANK_UINT8);
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (uint8_t)GAL_BLANK_UINT8)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (uint8_t)GAL_BLANK_UINT8)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_INT8:
       fmt = width ? "%*"PRId8 : "%"PRId8;
-      if(width) asprintf(&blank, fmt, width, (int8_t)GAL_BLANK_INT8);
-      else      asprintf(&blank, fmt,        (int8_t)GAL_BLANK_INT8);
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (int8_t)GAL_BLANK_INT8)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (int8_t)GAL_BLANK_INT8)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_UINT16:
       fmt = width ? "%*"PRIu16 : "%"PRIu16;
-      if(width) asprintf(&blank, fmt, width, (uint16_t)GAL_BLANK_UINT16);
-      else      asprintf(&blank, fmt,        (uint16_t)GAL_BLANK_UINT16);
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (uint16_t)GAL_BLANK_UINT16)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (uint16_t)GAL_BLANK_UINT16)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_INT16:
       fmt = width ? "%*"PRId16 : "%"PRId16;
-      if(width) asprintf(&blank, fmt, width, (int16_t)GAL_BLANK_INT16);
-      else      asprintf(&blank, fmt,        (int16_t)GAL_BLANK_INT16);
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (int16_t)GAL_BLANK_INT16)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (int16_t)GAL_BLANK_INT16)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_UINT32:
       fmt = width ? "%*"PRIu32 : "%"PRIu32;
-      if(width) asprintf(&blank, fmt, width, (uint32_t)GAL_BLANK_UINT32);
-      else      asprintf(&blank, fmt,        (uint32_t)GAL_BLANK_UINT32);
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (uint32_t)GAL_BLANK_UINT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (uint32_t)GAL_BLANK_UINT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_INT32:
       fmt = width ? "%*"PRId32 : "%"PRId32;
-      if(width) asprintf(&blank, fmt, width, (int32_t)GAL_BLANK_INT32);
-      else      asprintf(&blank, fmt,        (int32_t)GAL_BLANK_INT32);
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (int32_t)GAL_BLANK_INT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (int32_t)GAL_BLANK_INT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_UINT64:
       fmt = width ? "%*"PRIu64 : "%"PRIu64;
-      if(width) asprintf(&blank, fmt, width, (uint64_t)GAL_BLANK_UINT64);
-      else      asprintf(&blank, fmt,        (uint64_t)GAL_BLANK_UINT64);
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (uint64_t)GAL_BLANK_UINT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (uint64_t)GAL_BLANK_UINT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_INT64:
       fmt = width ? "%*"PRId64 : "%"PRId64;
-      if(width) asprintf(&blank, fmt, width, (int64_t)GAL_BLANK_INT64);
-      else      asprintf(&blank, fmt,        (int64_t)GAL_BLANK_INT64);
+      if(width)
+        {
+          if( asprintf(&blank, fmt, width, (int64_t)GAL_BLANK_INT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, fmt, (int64_t)GAL_BLANK_INT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_FLOAT32:
-      if(width) asprintf(&blank, "%*f", width,  (float)GAL_BLANK_FLOAT32);
-      else      asprintf(&blank, "%f",          (float)GAL_BLANK_FLOAT32);
+      if(width)
+        {
+          if( asprintf(&blank, "%*f", width,  (float)GAL_BLANK_FLOAT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, "%f", (float)GAL_BLANK_FLOAT32)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     case GAL_TYPE_FLOAT64:
-      if(width) asprintf(&blank, "%*f", width,  (double)GAL_BLANK_FLOAT64);
-      else      asprintf(&blank, "%f",          (double)GAL_BLANK_FLOAT64);
+      if(width)
+        {
+          if( asprintf(&blank, "%*f", width,  (double)GAL_BLANK_FLOAT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        {
+          if( asprintf(&blank, "%f", (double)GAL_BLANK_FLOAT64)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       break;
 
     default:
