@@ -3,9 +3,9 @@ Functions to work with FITS image data.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -39,6 +39,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/fits.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/blank.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/checkset.h>
 #include <gnuastro-internal/tableintern.h>
@@ -147,7 +148,10 @@ gal_fits_name_save_as_string(char *filename, char *hdu)
 {
   char *name;
   if( gal_fits_name_is_fits(filename) )
-    asprintf(&name, "%s (hdu: %s)", filename, hdu);
+    {
+      if( asprintf(&name, "%s (hdu: %s)", filename, hdu)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
   else gal_checkset_allocate_copy(filename, &name);
   return name;
 }
@@ -671,7 +675,8 @@ gal_fits_hdu_open(char *filename, char *hdu, int iomode)
   fitsfile *fptr;
 
   /* Add hdu to filename: */
-  asprintf(&ffname, "%s[%s#]", filename, hdu);
+  if( asprintf(&ffname, "%s[%s#]", filename, hdu)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
   /* Open the FITS file: */
   if( fits_open_file(&fptr, ffname, iomode, &status) )
@@ -839,7 +844,7 @@ gal_fits_key_img_blank(uint8_t type)
      type. */
   if(tocopy)
     {
-      out = gal_data_malloc_array(type, 1, __func__, "out");
+      out = gal_pointer_allocate(type, 1, 0, __func__, "out");
       memcpy(out, tocopy, gal_type_sizeof(type));
     }
 
@@ -951,8 +956,8 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
            set the size and ndim to 1. But first allocate dsize if it
            wasn't already allocated. */
         if(tmp->dsize==NULL)
-          tmp->dsize=gal_data_malloc_array(GAL_TYPE_SIZE_T, 1, __func__,
-                                           "tmp->dsize");
+          tmp->dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, 1, 0, __func__,
+                                          "tmp->dsize");
         tmp->ndim=tmp->size=tmp->dsize[0]=1;
 
         /* When the type is a string, `tmp->array' is an array of pointers
@@ -962,12 +967,12 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
         switch(tmp->type)
           {
           case GAL_TYPE_STRING:
-            errno=0;
             tmp->array=strarray=( tmp->array
                                   ? tmp->array
-                                  : gal_data_malloc_array(tmp->type, 1,
-                                                          __func__,
-                                                          "tmp->array") );
+                                  : gal_pointer_allocate(tmp->type, 1, 0,
+                                                         __func__,
+                                                         "tmp->array") );
+            errno=0;
             valueptr=strarray[0]=malloc(FLEN_VALUE * sizeof *strarray[0]);
             if(strarray[0]==NULL)
               error(EXIT_FAILURE, errno, "%s: %zu bytes for strarray[0]",
@@ -977,9 +982,9 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
           default:
             tmp->array=valueptr=( tmp->array
                                   ? tmp->array
-                                  : gal_data_malloc_array(tmp->type, 1,
-                                                          __func__,
-                                                          "tmp->array") );
+                                  : gal_pointer_allocate(tmp->type, 1, 0,
+                                                         __func__,
+                                                         "tmp->array") );
           }
 
         /* Allocate space for the keyword comment if necessary.*/
@@ -1026,7 +1031,6 @@ gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
 
         /* Strings need to be cleaned (CFITSIO puts `'' around them with
            some (possiblly) extra space on the two ends of the string. */
-
       }
 }
 
@@ -1486,7 +1490,7 @@ gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize,
 
   /* Allocate the array to keep the dimension size and fill it in, note
      that its order is the opposite of naxes. */
-  *dsize=gal_data_malloc_array(GAL_TYPE_INT64, *ndim, __func__, "dsize");
+  *dsize=gal_pointer_allocate(GAL_TYPE_INT64, *ndim, 0, __func__, "dsize");
   for(i=0; i<*ndim; ++i)
     (*dsize)[i]=naxes[*ndim-1-i];
 
@@ -1502,8 +1506,7 @@ gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize,
 
 /* Read a FITS image HDU into a Gnuastro data structure. */
 gal_data_t *
-gal_fits_img_read(char *filename, char *hdu, size_t minmapsize,
-                  size_t hstartwcs, size_t hendwcs)
+gal_fits_img_read(char *filename, char *hdu, size_t minmapsize)
 {
   void *blank;
   long *fpixel;
@@ -1540,7 +1543,7 @@ gal_fits_img_read(char *filename, char *hdu, size_t minmapsize,
      problem, the space will be written/read as 32-bit `long' any way,
      we'll just have a few empty bytes that will be freed anyway at the end
      of this function. */
-  fpixel=gal_data_malloc_array(GAL_TYPE_INT64, ndim, __func__, "fpixel");
+  fpixel=gal_pointer_allocate(GAL_TYPE_INT64, ndim, 0, __func__, "fpixel");
   for(i=0;i<ndim;++i) fpixel[i]=1;
 
 
@@ -1548,6 +1551,8 @@ gal_fits_img_read(char *filename, char *hdu, size_t minmapsize,
   img=gal_data_alloc(NULL, type, ndim, dsize, NULL, 0, minmapsize,
                      name, unit, NULL);
   blank=gal_blank_alloc_write(type);
+  if(name) free(name);
+  if(unit) free(unit);
   free(dsize);
 
 
@@ -1559,10 +1564,6 @@ gal_fits_img_read(char *filename, char *hdu, size_t minmapsize,
   free(blank);
 
 
-  /* Read the WCS structure (if the FITS file has any). */
-  img->wcs=gal_wcs_read_fitsptr(fptr, hstartwcs, hendwcs, &img->nwcs);
-
-
   /* Close the input FITS file. */
   fits_close_file(fptr, &status);
   gal_fits_io_error(status, NULL);
@@ -1581,13 +1582,12 @@ gal_fits_img_read(char *filename, char *hdu, size_t minmapsize,
    used to convert the input file to the desired type. */
 gal_data_t *
 gal_fits_img_read_to_type(char *inputname, char *hdu, uint8_t type,
-                          size_t minmapsize, size_t hstartwcs,
-                          size_t hendwcs)
+                          size_t minmapsize)
 {
   gal_data_t *in, *converted;
 
   /* Read the specified input image HDU. */
-  in=gal_fits_img_read(inputname, hdu, minmapsize, hstartwcs, hendwcs);
+  in=gal_fits_img_read(inputname, hdu, minmapsize);
 
   /* If the input had another type, convert it to float. */
   if(in->type!=type)
@@ -1616,7 +1616,7 @@ gal_fits_img_read_kernel(char *filename, char *hdu, size_t minmapsize)
 
   /* Read the image as a float and if it has a WCS structure, free it. */
   kernel=gal_fits_img_read_to_type(filename, hdu, GAL_TYPE_FLOAT32,
-                                   minmapsize, 0, 0);
+                                   minmapsize);
   if(kernel->wcs) { wcsfree(kernel->wcs); kernel->wcs=NULL; }
 
   /* Check if the size along each dimension of the kernel is an odd
@@ -1681,9 +1681,10 @@ gal_fits_img_write_to_ptr(gal_data_t *input, char *filename)
   hasblank=gal_blank_present(towrite, 0);
 
   /* Allocate the naxis area. */
-  naxes=gal_data_malloc_array( ( sizeof(long)==8
-                                 ? GAL_TYPE_INT64
-                                 : GAL_TYPE_INT32 ), ndim, __func__, "naxes");
+  naxes=gal_pointer_allocate( ( sizeof(long)==8
+                                ? GAL_TYPE_INT64
+                                : GAL_TYPE_INT32 ), ndim, 0, __func__,
+                              "naxes");
 
 
   /* Open the file for writing */
@@ -2344,65 +2345,188 @@ gal_fits_tab_info(char *filename, char *hdu, size_t *numcols,
 
 
 
-/* Read the column indexs given in the `indexll' linked list from a FITS
-   table into a linked list of data structures, note that this is a
-   low-level function, so the output data linked list is the inverse of the
-   input indexs linked list. */
+/* Read CFITSIO un-readable (INF, -INF or NAN) floating point values in
+   FITS ASCII tables. */
+static void
+fits_tab_read_ascii_float_special(char *filename, char *hdu, fitsfile *fptr,
+                                  gal_data_t *out, size_t colnum,
+                                  size_t numrows, size_t minmapsize)
+{
+  double tmp;
+  char **strarr;
+  char *tailptr;
+  gal_data_t *strrows;
+  size_t i, colwidth=50;
+  int anynul=0, status=0;
+
+  /* Allocate the dataset to keep the string values. */
+  strrows=gal_data_alloc(NULL, GAL_TYPE_STRING, 1, &numrows, NULL, 0,
+                         minmapsize, NULL, NULL, NULL);
+
+  /* Allocate the space to keep the string values. */
+  strarr=strrows->array;
+  for(i=0;i<numrows;++i)
+    {
+      errno=0;
+      strarr[i]=calloc(colwidth, sizeof *strarr[i]);
+      if(strarr[i]==NULL)
+        error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for "
+              "strarr[%zu]", __func__, colwidth * sizeof *strarr[i], i);
+    }
+
+  /* Read the column as a string. */
+  fits_read_col(fptr, TSTRING, colnum, 1, 1, out->size, NULL, strrows->array,
+                &anynul, &status);
+  gal_fits_io_error(status, NULL);
+
+  /* Convert the strings to float. */
+  for(i=0;i<numrows;++i)
+    {
+      /* Parse the string. */
+      tmp=strtod(strarr[i], &tailptr);
+      if(tailptr==strarr[i])
+        error(EXIT_FAILURE, 0, "%s (hdu %s): couldn't parse row %zu of "
+              "column %zu (with value `%s') as a floating point number",
+              filename, hdu, i+1, colnum, strarr[i]);
+
+      /* Write it into the output dataset. */
+      if(out->type==GAL_TYPE_FLOAT32)
+        ((float *)(out->array))[i]=tmp;
+      else
+        ((double *)(out->array))[i]=tmp;
+    }
+
+  /* Clean up. */
+  gal_data_free(strrows);
+}
+
+
+
+
+
+/* Read the column indexs into a dataset. */
 gal_data_t *
 gal_fits_tab_read(char *filename, char *hdu, size_t numrows,
                   gal_data_t *allcols, gal_list_sizet_t *indexll,
-                  int minmapsize)
+                  size_t minmapsize)
 {
   size_t i=0;
   void *blank;
-  size_t dsize;
   char **strarr;
   fitsfile *fptr;
   gal_data_t *out=NULL;
-  int status=0, anynul=0;
   gal_list_sizet_t *ind;
+  int isfloat, status=0, anynul=0, hdutype;
 
-  /* Open the FITS file */
-  fptr=gal_fits_hdu_open_format(filename, hdu, 1);
-
-  /* Pop each index and read/store the array. */
-  for(ind=indexll; ind!=NULL; ind=ind->next)
+  /* We actually do have columns to read. */
+  if(numrows)
     {
-      /* Allocate the necessary data structure (including the array) for
-         this column. */
-      dsize=numrows;
-      gal_list_data_add_alloc(&out, NULL, allcols[ind->v].type, 1, &dsize,
-                              NULL, 0, minmapsize, allcols[ind->v].name,
-                              allcols[ind->v].unit, allcols[ind->v].comment);
+      /* Open the FITS file */
+      fptr=gal_fits_hdu_open_format(filename, hdu, 1);
 
-      /* For a string column, we need an allocated array for each element,
-         even in binary values. This value should be stored in the
-         disp_width element of the data structure, which is done
-         automatically in `gal_fits_table_info'. */
-      if(out->type==GAL_TYPE_STRING)
-        for(i=0;i<numrows;++i)
-          {
-            strarr=out->array;
-            errno=0;
-            strarr[i]=calloc(allcols[ind->v].disp_width+1, sizeof *strarr[i]);
-            if(strarr[i]==NULL)
-              error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for "
-                    "strarr[%zu]", __func__,
-                    (allcols[ind->v].disp_width+1) * sizeof *strarr[i], i);
-          }
+      /* See if its a Binary or ASCII table (necessary for floating point
+         blank values). */
+      if (fits_get_hdu_type(fptr, &hdutype, &status) )
+        gal_fits_io_error(status, NULL);
 
-      /* Allocate a blank value for the given type and read/store the
-         column using CFITSIO. Afterwards, free the blank value. */
-      blank=gal_blank_alloc_write(out->type);
-      fits_read_col(fptr, gal_fits_type_to_datatype(out->type), ind->v+1,
-                    1, 1, out->size, blank, out->array, &anynul, &status);
+      /* Pop each index and read/store the array. */
+      for(ind=indexll; ind!=NULL; ind=ind->next)
+        {
+          /* Allocate the necessary data structure (including the array)
+             for this column. */
+          gal_list_data_add_alloc(&out, NULL, allcols[ind->v].type, 1,
+                                  &numrows, NULL, 0, minmapsize,
+                                  allcols[ind->v].name, allcols[ind->v].unit,
+                                  allcols[ind->v].comment);
+
+          /* For a string column, we need an allocated array for each element,
+             even in binary values. This value should be stored in the
+             disp_width element of the data structure, which is done
+             automatically in `gal_fits_table_info'. */
+          if(out->type==GAL_TYPE_STRING)
+            {
+              strarr=out->array;
+              for(i=0;i<numrows;++i)
+                {
+                  errno=0;
+                  strarr[i]=calloc(allcols[ind->v].disp_width+1,
+                                   sizeof *strarr[i]);
+                  if(strarr[i]==NULL)
+                    error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for "
+                          "strarr[%zu]", __func__,
+                          (allcols[ind->v].disp_width+1) * sizeof *strarr[i],
+                          i);
+                }
+            }
+
+          /* Allocate a blank value for the given type and read/store the
+             column using CFITSIO. Note that for binary tables, we only
+             need blank values for integer types. For binary floating point
+             types, the FITS standard defines blanks as NaN (same as almost
+             any other software like Gnuastro). However if a blank value is
+             specified, CFITSIO will convert other special numbers like
+             `inf' to NaN also. We want to be able to distringuish `inf'
+             and NaN here, so for floating point types in binary tables, we
+             won't define any blank value. In ASCII tables, CFITSIO doesn't
+             read the `NAN' values (that it has written itself) unless we
+             specify a blank pointer/value. */
+          isfloat = ( out->type==GAL_TYPE_FLOAT32
+                      || out->type==GAL_TYPE_FLOAT64 );
+          blank = ( ( hdutype==BINARY_TBL && isfloat )
+                    ? NULL
+                    : gal_blank_alloc_write(out->type) );
+          fits_read_col(fptr, gal_fits_type_to_datatype(out->type), ind->v+1,
+                        1, 1, out->size, blank, out->array, &anynul, &status);
+
+          /* In the ASCII table format, CFITSIO might not be able to read
+             `INF' or `-INF'. In this case, it will set status to `BAD_C2D'
+             or `BAD_C2F'. So, we'll use our own parser for the column
+             values. */
+          if( hdutype==ASCII_TBL
+              && isfloat
+              && (status==BAD_C2D || status==BAD_C2F) )
+            {
+              fits_tab_read_ascii_float_special(filename, hdu, fptr, out,
+                                                ind->v+1, numrows,
+                                                minmapsize);
+              status=0;
+            }
+
+          /* Clean up and sanity check. */
+          if(blank) free(blank);
+          gal_fits_io_error(status, NULL);
+        }
+
+      /* Close the FITS file */
+      fits_close_file(fptr, &status);
       gal_fits_io_error(status, NULL);
-      free(blank);
     }
 
-  /* Close the FITS file */
-  fits_close_file(fptr, &status);
-  gal_fits_io_error(status, NULL);
+  /* There are no rows to read (`numrows==NULL'). Make an empty-sized
+     array. */
+  else
+    {
+      /* We are setting a 1-element array to avoid any allocation
+         errors. Then we are freeing the allocated spaces and correcting
+         the sizes. */
+      numrows=1;
+      for(ind=indexll; ind!=NULL; ind=ind->next)
+        {
+          /* Do the allocation. */
+          gal_list_data_add_alloc(&out, NULL, allcols[ind->v].type, 1,
+                                  &numrows, NULL, 0, minmapsize,
+                                  allcols[ind->v].name, allcols[ind->v].unit,
+                                  allcols[ind->v].comment);
+
+          /* Correct the array and sizes. */
+          out->size=0;
+          free(out->array);
+          free(out->dsize);
+          out->dsize=out->array=NULL;
+        }
+    }
+
+  /* Return the output. */
   return out;
 }
 
@@ -2494,8 +2618,10 @@ fits_table_prepare_arrays(gal_data_t *cols, size_t numcols, int tableformat,
   for(col=cols; col!=NULL; col=col->next)
     {
       /* Set the `ttype' and `tunit' values: */
-      asprintf(&ttype[i], "%s", col->name ? col->name : "");
-      asprintf(&tunit[i], "%s", col->unit ? col->unit : "");
+      if( asprintf(&ttype[i], "%s", col->name ? col->name : "")<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+      if( asprintf(&tunit[i], "%s", col->unit ? col->unit : "")<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
 
       /* FITS's TFORM depends on the type of FITS table, so work
@@ -2537,13 +2663,15 @@ fits_table_prepare_arrays(gal_data_t *cols, size_t numcols, int tableformat,
               case GAL_TYPE_INT32:
               case GAL_TYPE_UINT64:
               case GAL_TYPE_INT64:
-                asprintf(&tform[i], "%c%d", fmt[0], col->disp_width);
+                if( asprintf(&tform[i], "%c%d", fmt[0], col->disp_width)<0 )
+                  error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
                 break;
 
               case GAL_TYPE_FLOAT32:
               case GAL_TYPE_FLOAT64:
-                asprintf(&tform[i], "%c%d.%d", fmt[0], col->disp_width,
-                         col->disp_precision);
+                if( asprintf(&tform[i], "%c%d.%d", fmt[0], col->disp_width,
+                             col->disp_precision)<0 )
+                  error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
                 break;
 
               default:
@@ -2561,9 +2689,15 @@ fits_table_prepare_arrays(gal_data_t *cols, size_t numcols, int tableformat,
           col->disp_width=fits_string_fixed_alloc_size(col);
           fmt[0]=gal_fits_type_to_bin_tform(col->type);
           if( col->type==GAL_TYPE_STRING )
-            asprintf(&tform[i], "%d%c", col->disp_width, fmt[0]);
+            {
+              if( asprintf(&tform[i], "%d%c", col->disp_width, fmt[0])<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+            }
           else
-            asprintf(&tform[i], "%c", fmt[0]);
+            {
+              if( asprintf(&tform[i], "%c", fmt[0])<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+            }
           break;
 
         default:
@@ -2599,7 +2733,8 @@ fits_write_tnull_tcomm(fitsfile *fptr, gal_data_t *col, int tableformat,
     case GAL_TABLE_FORMAT_AFITS:
 
       /* Print the keyword and value. */
-      asprintf(&keyname, "TNULL%zu", colnum);
+      if( asprintf(&keyname, "TNULL%zu", colnum)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       blank=gal_blank_as_string(col->type, col->disp_width);
 
       /* When in exponential form (`tform' starting with `E'), CFITSIO
@@ -2627,7 +2762,8 @@ fits_write_tnull_tcomm(fitsfile *fptr, gal_data_t *col, int tableformat,
           && col->type!=GAL_TYPE_STRING )
         {
           blank=gal_blank_alloc_write(col->type);
-          asprintf(&keyname, "TNULL%zu", colnum);
+          if( asprintf(&keyname, "TNULL%zu", colnum)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
           fits_write_key(fptr, gal_fits_type_to_datatype(col->type),
                          keyname, blank, "blank value for this column",
                          &status);
@@ -2645,8 +2781,10 @@ fits_write_tnull_tcomm(fitsfile *fptr, gal_data_t *col, int tableformat,
   /* Write the comments if there is any. */
   if(col->comment)
     {
-      asprintf(&keyname, "TCOMM%zu", colnum);
-      asprintf(&bcomment, "comment for field %zu", colnum);
+      if( asprintf(&keyname, "TCOMM%zu", colnum)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+      if( asprintf(&bcomment, "comment for field %zu", colnum)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       fits_write_key(fptr, TSTRING, keyname, col->comment,
                      bcomment, &status);
       gal_fits_io_error(status, NULL);
@@ -2689,7 +2827,7 @@ gal_fits_tab_write(gal_data_t *cols, gal_list_str_t *comments,
   fptr=gal_fits_open_to_write(filename);
 
 
-  /* prepare necessary arrays and if integer type columns have blank
+  /* Prepare necessary arrays and if integer type columns have blank
      values, write the TNULLn keywords into the FITS file. */
   fits_table_prepare_arrays(cols, numcols, tableformat,
                             &tform, &ttype, &tunit);
