@@ -2,9 +2,9 @@
 Function to parse options and configuration file values.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -195,6 +195,7 @@ struct gal_options_common_params
   size_t            numthreads; /* Number of threads to use.              */
   size_t            minmapsize; /* Minimum bytes necessary to use mmap.   */
   uint8_t                  log; /* Make a log file.                       */
+  char            *onlyversion; /* Redundant, kept/set for generality.    */
 
   /* Configuration files. */
   uint8_t          printparams; /* To print the full list of parameters.  */
