@@ -3,9 +3,9 @@ blank -- Deal with blank values in a datasets
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -95,6 +95,9 @@ gal_blank_initialize(gal_data_t *input);
 char *
 gal_blank_as_string(uint8_t type, int width);
 
+int
+gal_blank_is(void *pointer, uint8_t type);
+
 int
 gal_blank_present(gal_data_t *input, int updateflag);
 
