@@ -4,7 +4,7 @@
 # (in the Installing gnuastro section).
 #
 # Original author:
-#     Mohammad Akhlaghi <akhlaghi@gnu.org>
+#     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
 #
 # Copying and distribution of this file, with or without modification,
@@ -41,18 +41,10 @@ source=$topsrc/tests/$prog/simpleio.c
 #
 #   - The input data was not made (for example the test that created the
 #     data file failed).
-#
-#   - Gnuastro ships with its own version of Libtool for the building of
-#     the libraries and programs. But here, we want to test the user's
-#     libtool (and how it works with BuildProgram). So if libtool wasn't
-#     found at configure time, we need to skip this test.
 if [ ! -f $execname ]; then echo "$execname not created.";  exit 77; fi
 if [ ! -f $img      ]; then echo "$img does not exist.";    exit 77; fi
 if [ ! -f $source   ]; then echo "$source does not exist."; exit 77; fi
-if [ "x$hasgnulibtool" != "xyes" ];then
-    echo "GNU libtool not present.";
-    exit 77;
-fi
+
 
 
 
@@ -65,4 +57,9 @@ fi
 #
 # Except for `gnuastro/config.h', all headers are installed in
 # `$topsrc/lib' and `gnuastro/config.h' is in "../lib/"
-$execname $source $img 1 --la=../lib/libgnuastro.la -I$topsrc/lib -I../lib/
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $source $img 1 --la=../lib/libgnuastro.la \
+                              -I$topsrc/lib -I../lib/
